
var CrxfsBackupClient = {

    server: "127.0.0.1:8080",

	//TEMPORARY or PERSISTENT
	type : window.PERSISTENT,
	size : 1024 * 1024,

    getAll: function () {
        var backup = this;

        backup.xhr.get(
            "http://" + this.server,
            null,
            function (res, status) {
				var data = JSON.parse(res);
                getR(data.results, function () {
                    console.log("done");
                });
            }
        );

        function getR(es, callback) {
            (function () {
                var callee = arguments.callee;
                var entry = es.shift();
                if (!entry) {
                    callback();
                    return;
                }
                getF(entry, function () {
                    setTimeout(callee, 10);
                });
            })();
        }

        function getF(path, callback) {
            backup.xhr.get(
                "http://" + backup.server + path,
                null,
                function (res, status) {
					var data = JSON.parse(res);
                    var fs = new CrxfsBackupClient.FileSystem(backup.type, backup.size);
                    fs.writeDataURL(path, data.dataURL, function(){
                        console.log("get:" + path);
                        callback();
                    });
                }
            );
        }
    },

    
    postAll: function () {
        var backup = this;

        var fs = new CrxfsBackupClient.FileSystem(backup.type, backup.size);
        fs.getDir("/", function (dirEntry) {
            var files = [];
            postFiles(dirEntry, files, function () {
                console.log(files.length + " files");
                postR(files, function () {
                    console.log("done");
                });
            });
        });

        function postFiles(dir, files, callback) {
            var reader = dir.createReader();
            reader.readEntries(
                function (entries) {
                    var es = [];
                    for (var i = 0; i < entries.length; i++) {
                        es.push(entries[i]);
                    }
                    (function () {
                        var callee = arguments.callee;

                        var entry = es.shift();
                        if (!entry) {
                            callback();
                            return;
                        }
                        if (entry.isDirectory) {
                            //console.log("dir:" + entry.toURL());
                            postFiles(entry, files, function () {
                                setTimeout(callee, 1);
                            });
                        } else {
                            //console.log("file:" + entry.toURL());
                            files.push(entry);
                            setTimeout(callee, 1);
                        }
                    })();
                }
            );
        }

        function postR(es, callback) {
            (function () {
                var callee = arguments.callee;
                var entry = es.shift();
                if (!entry) {
                    callback();
                    return;
                }
                postF(entry, function () {
                    setTimeout(callee, 10);
                });
            })();
        }

        function postF(entry, callback) {
            var path = entry.fullPath + "";
            var fs = new CrxfsBackupClient.FileSystem(backup.type, backup.size);
            fs.readDataURL(path, function (dataURL) {
                var param = { path: path, dataURL: dataURL };
                backup.xhr.post(
                    "http://" + backup.server,
                    param,
                    function (res, status) {
                        console.log("post:" + path);
                        callback();
                    }
                );
            });
        }
    },

	xhr: {
		post: function(url, param, callback, errback){
			var xhr = new XMLHttpRequest();
	        xhr.open("POST", url, true);
        	xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
			xhr.onload = function () {
				if (xhr.status == 200) {
					callback(xhr.responseText, xhr.status);
				}else{
					if(errback) errback();
				}
			}
			xhr.onerror = function(){
				if(errback) errback();
			}
			xhr.send(this.param2str(param));
		},

		get: function(url, param, callback, errback){
			if(param){
				url = url + "?" + this.param2str(param);
			}
			var xhr = new XMLHttpRequest();
	        xhr.open("GET", url, true);
			xhr.onload = function () {
				if (xhr.status == 200) {
					callback(xhr.responseText, xhr.status);
				}else{
					if(errback) errback();
				}
			}
			xhr.onerror = function(){
				if(errback) errback();
			}
			xhr.send();
		},

		param2str: function(obj) {
	        var str = "";
	        for (var i in obj) {
	            var v = obj[i];
	            if (str) { str += "&"; };
	            str += i + "=" + encodeURIComponent(v);
	        }
	        return str;
	    }
	},

	FileSystem: function (type, size) {

	    this.type = type,
	    this.size = size,

	    this.getFile = function (file, successCallback, errorCallback, options) {
	        options = options ? options : { "create": false };

	        webkitRequestFileSystem(this.type, this.size, function (fileSystem) {
	            fileSystem.root.getFile(file, options, function (fileEntry) {
	                successCallback(fileEntry);
	            }, errorCallback);
	        }, errorCallback);
	    },

	    this.getDir = function (dir, successCallback, errorCallback, options) {
	        options = options ? options : { "create": false };

	        webkitRequestFileSystem(this.type, this.size, function (fileSystem) {
	            if (dir == "/") {
	                successCallback(fileSystem.root);
	            } else {
	                fileSystem.root.getDirectory(dir, options, function (dirEntry) {
	                    successCallback(dirEntry);
	                }, errorCallback);
	            }
	        }, errorCallback);
	    },

	    this.makeDir = function (dir, successCallback, errorCallback) {
	        this.getDir(dir, successCallback, errorCallback, { "create": true });
	    },

	    this.makeDirR = function (dir, successCallback, errorCallback) {
	        var folders = dir.split("/");
	        this.getDir("/", function (dirEntry) {
	            (function (rootDirEntry, folders) {
	                var callee = arguments.callee;
	                if (folders[0] == '.' || folders[0] == '') {
	                    folders = folders.slice(1);
	                }
	                if (folders.length == 0) {
	                    successCallback(rootDirEntry);
	                } else {
	                    rootDirEntry.getDirectory(folders[0], { create: true }, function (dirEntry) {
	                        if (folders.length) {
	                            callee(dirEntry, folders.slice(1));
	                        } else {
	                            successCallback(dirEntry);
	                        }
	                    }, errorCallback);
	                }
	            })(dirEntry, folders);
	        }, errorCallback);
	    },

	    this.writeBlob = function (file, blob, successCallback, errorCallback) {
	        var dirs = file.split("/");
	        dirs.pop();
	        var dir = dirs.join("/");

	        var fs = this;
	        fs.makeDirR(dir, function (dirEntry) {
	            fs.getFile(file, function (fileEntry) {
	                fileEntry.createWriter(function (fileWriter) {
	                    fileWriter.onwrite = function (e) {
	                        successCallback(fileEntry);
	                    };
	                    fileWriter.onerror = function (e) {
	                        errorCallback(e);
	                    };
	                    fileWriter.write(blob);
	                }, errorCallback);
	            }, errorCallback, { 'create': true });
	        }, errorCallback);
	    },

	    this.readBlob = function (file, successCallback, errorCallback) {
	        this.getFile(file, function (fileEntry) {
	            fileEntry.file(function (file) {
	                var blob = file.slice(0, file.size, file.type);
	                successCallback(blob);
	            });
	        }, errorCallback);
	    },

	    this.writeDataURL = function (file, dataURL, successCallback, errorCallback) {
	        var set = dataURL.split(";base64,", 2);
	        var mime = set[0].substr(5);
	        var data = set[1];
	        var raw = window.atob(data);
	        var uInt8Array = new Uint8Array(raw.length);

	        for (var i = 0; i < raw.length; ++i) {
	            uInt8Array[i] = raw.charCodeAt(i);
	        }

	        var blob = new Blob([uInt8Array], { "type": mime });

	        this.writeBlob(file, blob, successCallback, errorCallback);
	    },

	    this.readDataURL = function (file, successCallback, errorCallback) {
	        this.getFile(file, function (fileEntry) {
	            fileEntry.file(function (file) {
	                var reader = new FileReader();
	                reader.onload = function (e) {
	                    successCallback(e.target.result);
	                };
	                reader.onerror = function (e) {
	                    errorCallback(e);
	                };
	                reader.readAsDataURL(file);
	            });
	        }, errorCallback);
	    },

	    this.removeFile = function (file, successCallback, errorCallback) {
	        this.getFile(file, function (fileEntry) {
	            fileEntry.remove(function () {
	                successCallback();
	            }, errorCallback);
	        }, errorCallback);
	    }

	    this.removeDir = function (dir, successCallback, errorCallback) {
	        this.getDir(dir, function (dirEntry) {
	            dirEntry.remove(function () {
	                successCallback();
	            }, errorCallback);
	        }, errorCallback);
	    }

	    this.removeDirR = function (dir, successCallback, errorCallback) {
	        this.getDir(dir, function (dirEntry) {
	            dirEntry.removeRecursively(function () {
	                successCallback();
	            }, errorCallback);
	        }, errorCallback);
	    }

	    this.getUrl = function (file, successCallback, errorCallback) {
	        this.getFile(file, function (fileEntry) {
	            successCallback(fileEntry.toURL());
	        }, errorCallback);
	    }

	    this.getBlobUrl = function (file, successCallback, errorCallback) {
	        this.getFile(file, function (fileEntry) {
	            fileEntry.file(function (file) {
	                successCallback(webkitURL.createObjectURL(file));
	            });
	        }, errorCallback);
	    }
	}

}