var sys = require ('sys'),
url = require('url'),
http = require('http'),
fs = require("fs"),
path = require('path'),
qs = require('querystring');

var setting = {
	port : 8080,
	docRoot : "./www"
};

var mime = {
    "txt"  : "text/plain",
	"html" : "text/html",
	"css"  : "text/css",
	"js"   : "text/javascript",
	"zip"  : "application/zip",
	"gif"  : "image/gif",
	"jpeg" : "image/jpeg",
	"jpg"  : "image/jpeg",
    "png"  : "image/png",
	"ico"  : "image/x-icon",
	"json" : "application/json"
}

http.createServer(function (req, res) {
	if(req.method=='POST') {
		onPost(req, res);
	}
	else if(req.method=='GET') {
		onGet(req, res);
	}
}).listen(setting.port);
console.log('Server running at http://127.0.0.1:'+setting.port);

function onPost(req, res){
	var body='';
	var pathname = url.parse(req.url).pathname;
	req.setEncoding("utf8");
	req.addListener('data', function (data) {
		body +=data;
	});
	req.addListener('end',function(){
		if(body){
    		var POST =  qs.parse(body);
			onPostData(req, res, POST);
		}
		res.writeHead(200, {'Content-Type': 'text/plain'});
		res.end('200 OK\n');
	});
}

function onPostData(req, res, post){
	if(!post.path || !post.dataURL){
		return;
	}
	var filepath = post.path;
	if(filepath.substr(0,1)!="/"){
		filepath = "/" + filepath;
	}
	filepath = setting.docRoot + filepath;

	path.exists(filepath, function(exists) {
		if (!exists) {
			saveDataURL(filepath, post.dataURL);
			console.log("post:"+post.path);
		}else{
			console.log("exists:"+post.path);
		}
	});
}

function onGet(req, res){
	var pathname = url.parse(req.url).pathname;
	var url_parts = url.parse(req.url,true);
	var GET = url_parts.query;

	if(pathname=="/"){
		walkJSON(setting.docRoot, function(json){
			sendResponse(res, 200, mime["json"], JSON.stringify(json));
			console.log(json["results"].length+" files");
		});
	}else{
		var ext = path2ext(pathname);
		if(mime[ext]){
			readFile(pathname, function(data){
				if(data){
					var dataURL = base64Encode(mime[ext], data);
					var obj = {"path":pathname, "dataURL":dataURL};
					sendResponse(res, 200, mime["json"], JSON.stringify(obj));
					console.log("get:" + pathname);
				}else{
					sendResponse(res, 404, mime[ext], "");
					console.log("not found:" + pathname);
				}
			});
		}else{
			sendResponse(res, 403, mime[ext], "");
			console.log("unknown mime type [" + ext + "]:" + pathname);
		}
	}

	function path2ext(pathname){
		var ext = pathname.match(/\.([a-z0-9]+)$/i);
		return ext[1];
	}

	function walkJSON(dir, callback){
		walk(dir, function(err, results){
			var arr = [];
			for(var i=0;i<results.length;i++){
				arr.push(results[i].substr(setting.docRoot.length));
			}
			var obj = {"results":arr};
			callback(obj);
		});
	}

}

function base64Encode(contentType, data){
	var prefix = "data:" + contentType + ";base64,";
	var base64 = data.toString('base64');
	return prefix + base64;
}

function sendResponse(res, statusCode, contentType, body){
	res.writeHead(statusCode, {
		'Content-Type': contentType,
		'Content-Length': body.length
	});
	res.end(body);
}

function readFile(pathname, callback){
	pathname = setting.docRoot + pathname;
	path.exists(pathname, function(exists) {
		if (!exists) {
			callback("");
		}else{
			fs.readFile(pathname, function (err, data) {
				if(err){
					callback("");
				}else{
					callback(data);
				}
			});
		}
	});
}

//http://blog.livedoor.jp/aki_mana/archives/6127891.html
var parseDataURL = function( dataURL ){
	var rslt = {
		mediaType : null,
		encoding : null,
		isBase64 : null,
		data : null
	};
	if( /^data:([^;]+)(;charset=([^,;]+))?(;base64)?,(.*)/.test( dataURL ) ) {
		rslt.mediaType = RegExp.$1 || 'text/plain';
		rslt.encoding  = RegExp.$3 || 'US-ASCII';
		rslt.isBase64  = String(RegExp.$4) === ';base64';
		rslt.data      = RegExp.$5;
	}
	return rslt;
}

var saveDataURL = function( filepath, dataurl ){
	var info = parseDataURL( dataurl );
	var chunk = info.isBase64
		? new Buffer(info.data, 'base64')
		: new Buffer( unescape( info.data ), 'binary');

	var obj = getDirFileObject(filepath);
	mkdirp(obj.dir, function(){
		var newFile = fs.createWriteStream( filepath );
		newFile.write( chunk );
		newFile.end();
	});
}


function getDirFileObject(path){
	var dirs = path.split("/");
	var file = dirs.pop();
	var dir = dirs.join("/");
	return {"dir":dir, "file":file};
}

function mkdirp(dir, callback){

	if(!dir) return;

	var dirs = dir.split("/");
	if(dirs[0]=="."){
		dirs.shift();
	}

	(function(dirs, i){
		var callee = arguments.callee;
		if(dirs.length<=i){
			callback();
			return;
		};
		var rpath = dirs.slice(0, i+1).join("/");
		path.exists(rpath, function(exists) {
			if (!exists) {
				fs.mkdir(rpath, 0777, function (err) {
					if(!err){
						callee(dirs, i+1);
					}
				});
			}else{
				callee(dirs, i+1);
			}
		});
	})(dirs, 0);

}

//http://stackoverflow.com/questions/5827612/node-js-fs-readdir-recursive-directory-search
function walk(dir, done) {
  var results = [];
  fs.readdir(dir, function(err, list) {
    if (err) return done(err);
    var i = 0;
    (function next() {
      var file = list[i++];
      if (!file) return done(null, results);
      file = dir + '/' + file;
      fs.stat(file, function(err, stat) {
        if (stat && stat.isDirectory()) {
          walk(file, function(err, res) {
            results = results.concat(res);
            next();
          });
        } else {
          results.push(file);
          next();
        }
      });
    })();
  });
};
